@@ -22,7 +22,7 @@ kernel_path=`echo "$kernel_path" | sed -e 's/\/[^/]*$//'`
 relative=${orig:$((${#kernel_path} + 1))};
 
 if ! [ -a $new ]; then
-	echo "$new not found" >&2;
+	echo "$new not found" > /dev/stderr
 	exit 2;
 fi
 
