@@ -42,9 +42,6 @@ int lv_check_consistency ( lv_t *lv) {
 
    debug_enter ( "lv_check_consistency -- CALLED\n");
 
-   /* Don't barf on broken LVM1.1 created metadata */
-   if (lv->lv_status > 2) lv->lv_status = 1;
-
    if ( lv == NULL)                                   ret = -LVM_EPARAM;
    else if ( lv_check_name ( lv->lv_name) != 0)       ret = -LVM_ELV_LVNAME;
    else if ( vg_check_name ( lv->vg_name) != 0)       ret = -LVM_ELV_VGNAME;
