@@ -1,12 +1,13 @@
 /*
  * tools/lib/pv_get_size.c
  *
- * Copyright (C) 1997 - 1998  Heinz Mauelshagen, Sistina Software
+ * Copyright (C) 1997 - 2001  Heinz Mauelshagen, Sistina Software
  *
  * March,October 1997
  * May 1998
  * January,March,June,July,September 1999
  * January,February 2000
+ * June 2001
  *
  *
  * This LVM library is free software; you can redistribute it and/or
@@ -42,14 +43,56 @@
  *                 Andrea Arcangeli <andrea@suse.de>
  *    03/02/2000 - use debug_enter()/debug_leave()
  *    16/02/2000 - added partiton table error return code
+ *    15/03/2001 - Don't look for partitions on non-partition devices
+ *    09/04/2001 - Move pv_check_partitioned_whole into this file from
+ *                 program files
+ *    26/06/2001 - checked in pv_get_size() if no hit in the dir cache
+ *                 took place in order to avoid a segfault
  *
  */
 
 #include <liblvm.h>
 
+#ifndef WIN98_EXTENDED_PARTITION
+#define WIN98_EXTENDED_PARTITION 0x0f
+#endif
+
+/* assumes pv_name has already been validated */
+long long _pv_get_dev_size ( char *pv_name, struct partition *part_ptr)
+{
+   int pv_fd = -1;
+   ulong size;
+   long long ret = 1;
+
+   debug_enter ( "_pv_get_dev_size -- CALLED with %s and %p\n",
+                 pv_name, part_ptr);
+
+   pv_fd = open ( pv_name, O_RDONLY);
+   if ( pv_fd < 0)
+      ret = -LVM_EPV_GET_SIZE_OPEN;
+   else if ( ioctl ( pv_fd, BLKGETSIZE, &size) < 0)
+      ret = -LVM_EPV_GET_SIZE_IOCTL;
+   if ( pv_fd != -1) close ( pv_fd);
+
+   if ( part_ptr)
+      memset ( part_ptr, 0, sizeof ( struct partition));
+
+   if ( ret > 0) ret = size;
+   debug_leave ( "_pv_get_dev_size -- LEAVING with ret: %lu\n", ret);
+
+   return ret;
+}
 
 #ifndef __alpha__
-int pv_get_size ( char *dev_name, struct partition *part_ptr) {
+/* assumes pv_name has already been validated */
+
+long pv_get_size ( char *pv_name, struct partition *part_ptr) {
+   long long ret = pv_get_size_ll ( pv_name, part_ptr);
+   if ( ret > UINT_MAX) ret = -1LL;
+   return ( long) ret;
+}
+
+long long pv_get_size_ll ( char *pv_name, struct partition *part_ptr) {
    int i = 0;
    int dir_cache_count = 0;
    int extended_flag = 0;
@@ -57,63 +100,54 @@ int pv_get_size ( char *dev_name, struct partition *part_ptr) {
    int part_i = 0;
    int part_i_tmp = 0;
    int pv_handle = -1;
-   int ret = 0;
+   long long ret = 0;
    static char buffer[SECTOR_SIZE];
    loff_t offset = 0;
    loff_t extended_offset = 0;
    struct partition *part = ( struct partition *) ( buffer + 0x1be);
    unsigned short *s_buffer = ( unsigned short *) buffer;
-   char disk_dev_name[NAME_LEN] = { 0, };
+   char disk_dev_name[NAME_LEN];
    dev_t st_rdev = 0;
    dir_cache_t *dir_cache = NULL;
    dir_cache_t *cache_entry = NULL;
 
-#ifdef DEBUG
-   debug_enter ( "pv_get_size -- CALLED with %s and %X\n",
-                 dev_name, ( uint) part_ptr);
-#endif
+   debug_enter ( "pv_get_size -- CALLED with %s and %p\n",
+                 pv_name, part_ptr);
 
-   if ( dev_name == NULL ||
-        pv_check_name ( dev_name) < 0) {
+   if ( pv_name == NULL) {
       ret = -LVM_EPARAM;
-      goto pv_get_size_end;
+      goto pv_get_size_nopart;
    }
 
-   if ( ( part_i = pv_check_part ( dev_name))< 0) part_i = 0;
+   /* If we don't have a partitioned device or we are looking at the
+    * whole device rather than a partition, just return device size
+    */
+   if ( ( dir_cache_count = lvm_dir_cache ( &dir_cache)) < 1 ||
+        ( cache_entry = lvm_dir_cache_find ( pv_name)) == NULL ||
+        lvm_check_partitioned_dev ( cache_entry->st_rdev) == FALSE ||
+        ( part_i = MINOR ( cache_entry->st_rdev) %
+          lvm_partition_count ( cache_entry->st_rdev)) == 0) {
+      ret = _pv_get_dev_size ( pv_name, part_ptr);
+      goto pv_get_size_nopart;
+   }
 
    /* Figure out first st_rdev in case of SCSI, IDE etc. */
-   if ( ( dir_cache_count = lvm_dir_cache ( &dir_cache)) < 1) {
-      ret = -LVM_EPV_GET_SIZE_LVM_DIR_CACHE;
-      goto pv_get_size_end;
+   st_rdev = cache_entry->st_rdev - part_i;
+   for ( i = 0; i < dir_cache_count; i++) {
+      if ( dir_cache[i].st_rdev == st_rdev)
+            break;
    }
 
-   if ( ( cache_entry = lvm_dir_cache_find ( dev_name)) != NULL) {
-      if ( lvm_check_partitioned_dev ( cache_entry->st_rdev) == TRUE) {
-         st_rdev = cache_entry->st_rdev -
-                   ( cache_entry->st_rdev %
-                     lvm_partition_count ( cache_entry->st_rdev));
-         for ( i = 0; i < dir_cache_count; i++) {
-            if ( dir_cache[i].st_rdev == st_rdev)
-               break;
-         }
-      }
-   } else {
-      ret = -LVM_EPV_GET_SIZE_LVM_DIR_CACHE;
-      goto pv_get_size_end;
-   }
-
-   memset ( disk_dev_name, 0, sizeof ( disk_dev_name));
+   if ( i >= dir_cache_count) goto pv_get_size_nopart;
    strncpy ( disk_dev_name, dir_cache[i].dev_name, sizeof ( disk_dev_name) - 1);
 
    first = 1;
    if ( ( pv_handle = open ( disk_dev_name, O_RDONLY)) == -1) {
       ret = -LVM_EPV_GET_SIZE_OPEN;
    } else while ( ret == 0) {
-#ifdef DEBUG
       debug ( "pv_get_size -- BEFORE llseek %X:%X\n",
               ( uint) ( offset >> 32),
               ( uint) ( offset & 0xFFFFFFFF));
-#endif
       if ( llseek ( pv_handle, offset * SECTOR_SIZE, SEEK_SET) == -1) {
          ret = -LVM_EPV_GET_SIZE_LLSEEK;
          break;
@@ -125,69 +159,44 @@ int pv_get_size ( char *dev_name, struct partition *part_ptr) {
          break;
       } else ret = 0;
 
-
       /* walk thrugh primary partitions */
       if ( s_buffer[255] == 0xAA55) {
-         if ( st_rdev == cache_entry->st_rdev) {
-            ret = -LVM_EPV_GET_SIZE_PART;
-            goto pv_get_size_end;
-         }
          extended_flag = 0;
          for ( i = 0; i < 4; i++) {
-#ifdef DEBUG
             debug ( "pv_get_size -- part[%d].sys_ind: %1X  "
                     "part[%d].nr_sects: %d\n",
                     i, part[i].sys_ind, i, part[i].nr_sects);
-#endif
-         /* is it a welcome extended partition? */
+            /* is it a welcome extended partition? */
             if ( part[i].sys_ind == DOS_EXTENDED_PARTITION ||
-                 part[i].sys_ind == LINUX_EXTENDED_PARTITION
-#ifdef WIN98_EXTENDED_PARTITION
-                 || part[i].sys_ind == WIN98_EXTENDED_PARTITION
-#endif
-            ) {
-#ifdef DEBUG
-#ifdef WIN98_EXTENDED_PARTITION
+                 part[i].sys_ind == LINUX_EXTENDED_PARTITION ||
+                 part[i].sys_ind == WIN98_EXTENDED_PARTITION) {
                debug ( "pv_get_size -- DOS/LINUX/WIN98_EXTENDED_PARTITION\n");
-#else
-               debug ( "pv_get_size -- DOS/LINUX_EXTENDED_PARTITION\n");
-#endif
-#endif
                extended_flag = 1;
                offset = extended_offset + part[i].start_sect;
                if ( extended_offset == 0) extended_offset = part[i].start_sect;
                if ( first == 1) part_i_tmp++;
             } else if ( first == 1) {
-#ifdef DEBUG
                debug ( "pv_get_size -- first == 1\n");
-#endif
                if ( i == part_i) {
                   if ( part[i].sys_ind == 0) ret = -LVM_EPV_GET_SIZE_NO_PRIMARY;
                } else part_i_tmp++;
             } else if ( part[i].sys_ind != 0) {
-#ifdef DEBUG
                debug ( "pv_get_size -- first == 1\n");
-#endif
                part_i_tmp++;
             }
 
             if ( part_i == part_i_tmp) {
-#ifdef DEBUG
                debug ( "pv_get_size -- part_i == part_i_tmp\n");
-#endif
                if ( part[i].nr_sects == 0) {
-                  ioctl ( pv_handle, BLKGETSIZE, &ret);
-                  i = 0;
-                  memset ( &part[i], 0, sizeof ( struct partition));
+                  ret = _pv_get_dev_size ( pv_name, part_ptr);
+                  goto pv_get_size_nopart;
                } else ret = part[i].nr_sects;
                goto pv_get_size_end;
             }
          }
       } else {
-         ioctl ( pv_handle, BLKGETSIZE, &ret);
-         i = 0;
-         memset ( &part[i], 0, sizeof ( struct partition));
-         goto pv_get_size_end;
+         ret = _pv_get_dev_size ( pv_name, part_ptr);
+         goto pv_get_size_nopart;
       }
 
       first = 0;
@@ -196,28 +205,26 @@ int pv_get_size ( char *dev_name, struct partition *part_ptr) {
    }
 
 pv_get_size_end:
-   if ( part_ptr != NULL && ret > 0) memcpy ( part_ptr,
-                                              &part[i],
-                                              sizeof ( struct partition));
+   if ( part_ptr != NULL && ret > 0)
+      memcpy ( part_ptr, &part[i], sizeof ( struct partition));
 
+pv_get_size_nopart:
    if ( pv_handle != -1) close ( pv_handle);
 
-#ifdef DEBUG
-   debug_leave ( "pv_get_size -- LEAVING with ret: %d\n", ret);
-#endif
+   debug_leave ( "pv_get_size -- LEAVING with ret: %lu\n", ret);
    return ret;
 }
-#else
-int pv_get_size(char * dev_name, struct partition * dummy) {
-   int pv_fd, ret;
 
-   pv_fd = open(dev_name, O_RDONLY);
-   if (pv_fd < 0)
-      return -LVM_EPV_GET_SIZE_OPEN;
-   if (ioctl(pv_fd, BLKGETSIZE, &ret) < 0)
-      return -LVM_EPV_GET_SIZE_IOCTL;
-      if (close(pv_fd) < 0)
-         perror("pv_get_size alpha - close"), exit(1);
-   return ret;
+#else /* #ifndef __alpha__ */
+
+long long pv_get_size_ll ( char *pv_name, struct partition *dummy)
+{
+   return _pv_get_dev_size ( pv_name, dummy);
+}
+
+long pv_get_size ( char *pv_name, struct partition *dummy) {
+   long long ret = _pv_get_dev_size(pv_name, dummy);
+   if ( ret > UINT_MAX) ret = -1LL;
+   return ( long) ret;
 }
 #endif
