@@ -40,9 +40,8 @@ vg_t *vg_copy_from_disk ( vg_disk_t *vg_disk) {
    int i = 0;
    vg_t *vg_tmp = NULL;
 
-#ifdef DEBUG
    debug_enter ( "vg_copy_from_disk -- CALLED\n");
-#endif
+
    if ( vg_disk != NULL && ( vg_tmp = malloc ( sizeof ( vg_t))) != NULL) {
       memset ( vg_tmp, 0, sizeof ( vg_t));
       vg_tmp->vg_number = LVM_TO_CORE32 ( vg_disk->vg_number);
@@ -66,9 +65,7 @@ vg_t *vg_copy_from_disk ( vg_disk_t *vg_disk) {
       memcpy ( vg_tmp->vg_uuid, vg_disk->vg_uuid, UUID_LEN);
    }
 
-#ifdef DEBUG
    debug_leave ( "vg_copy_from_disk -- LEAVING\n");
-#endif
    return vg_tmp;
 }
 
@@ -76,9 +73,8 @@ vg_t *vg_copy_from_disk ( vg_disk_t *vg_disk) {
 vg_disk_t *vg_copy_to_disk ( vg_t *vg_core) {
    vg_disk_t *vg_tmp = NULL;
 
-#ifdef DEBUG
    debug_enter ( "vg_copy_to_disk -- CALLED\n");
-#endif
+
    if ( vg_core != NULL &&
         vg_check_consistency ( vg_core) == 0 &&
         ( vg_tmp = malloc ( sizeof ( vg_disk_t))) != NULL) {
@@ -101,8 +97,6 @@ vg_disk_t *vg_copy_to_disk ( vg_t *vg_core) {
       memcpy ( vg_tmp->vg_uuid, vg_core->vg_uuid, UUID_LEN);
    }
 
-#ifdef DEBUG
    debug_leave ( "vg_copy_to_disk -- LEAVING\n");
-#endif
    return vg_tmp;
 }
