@@ -1,10 +1,11 @@
 /*
  * tools/lib/vg_status_get_count.c
  *
- * Copyright (C) 1997 - 1999  Heinz Mauelshagen, Germany
+ * Copyright (C) 1997 - 2000  Heinz Mauelshagen, Sistina Software
  *
  * March,November 1997
  * January 1999
+ * January 2000
  *
  *
  * This LVM library is free software; you can redistribute it and/or
@@ -24,6 +25,13 @@
  *
  */
 
+/*
+ * Changelog
+ *
+ *    08/02/2000 - use debug_enter()/debug_leave()
+ *
+ */
+  
 #include <liblvm.h>
 
 
@@ -32,20 +40,16 @@ int vg_status_get_count ( void) {
    int lvm = -1;
    int ret = 0;
 
-#ifdef DEBUG
-   debug ( "vg_status_get_count -- CALLED\n");
-#endif
+   debug_enter ( "vg_status_get_count -- CALLED\n");
 
-   if ( ( lvm = open ( LVM_DEV, O_RDONLY)) == -1)
+   if ( ( lvm = open ( LVM_DEV, O_RDONLY)) == -1) {
       ret = -LVM_EVG_STATUS_GET_COUNT_OPEN;
-   else {
+   } else {
       ret = ioctl ( lvm, VG_STATUS_GET_COUNT, &count);
       close ( lvm);
    }
+   if ( ret == 0) ret = count;
 
-#ifdef DEBUG
-   debug ( "vg_status_get_count -- LEAVING\n");
-#endif
-   if ( ret == 0) return count;
-   else           return ret;
+   debug_leave ( "vg_status_get_count -- LEAVING with ret: %d\n", ret);
+   return ret;
 }
