@@ -79,8 +79,8 @@ int opt_ignore = 0;
 #define	LVMFILE_CHECK( chkfile) { \
    int file = -1; \
    if ( ( file = open ( chkfile, O_RDONLY)) == -1) { \
-      fprintf ( stderr, "%s -- ERROR: \"%s\" doesn't exist or is unreadable; " \
-                        "you may need to run vgscan\n\n", \
+      fprintf ( stderr, "%s -- ERROR: \"%s\" doesn't exist; " \
+                        "please run vgscan\n\n", \
                         cmd, chkfile); \
       return LVM_ELVMTAB; \
    } else close ( file); \
